#ifndef __MLS_COMMANDIMP_H__
#define __MLS_COMMANDIMP_H__

#include "define.h"
#include "panel.h"
#include "dialog.h"
#include "mainframe.h"
#include "mcd.h"

namespace MLS {

class CommandImp
{
	private:
		Panel *panel;

	public:
		CommandImp() : panel(NULL)	{ 	}
		~CommandImp() { }
		
		void SetPanel(Panel *p) { panel = p; }
		
		int  Run(const std::string &, bool = false);
		int  ParseAndRun(const string &p);
		
		void Default()			{ }
		
		void Key_Enter()		{ panel->Key_Enter();		}
		void Key_Left()			{ panel->Key_Left();	 	}
		void Key_Right() 		{ panel->Key_Right(); 		}
		void Key_Up () 			{ panel->Key_Up();			}
		void Key_Down () 		{ panel->Key_Down();		}
		void Key_Home () 		{ panel->Key_Home();		}
		void Key_End () 		{ panel->Key_End();			}
		void Key_PageUp() 		{ panel->Key_PageUp();		}
		void Key_PageDown() 	{ panel->Key_PageDown();	}
		
		// other implementations	
		void Remove()		    { panel->cmd_remove();	}
		
		void Editconfig()
		{
			Run(conf.extern_editor + " " + conf.home + ".mls/mls.cfg"); 
			
			conf.load((conf.home + ".mls/mls.cfg").c_str());
			panel->Refresh();
		}
		
		void Editcolorset()
		{
			Run(conf.extern_editor + " " + conf.home + ".mls/" + conf.colorset);
			conf.load((conf.home + ".mls/mls.cfg").c_str());
			
			panel->Refresh();
		}
		
		void About() 	{	panel->about();			}
		void Help()  	{	panel->cmd_help();		}
		void Exit()  	
		{
			int nYN = YNBox("Do you really want to quit the mls ?", 0);
			
			if (nYN == YN_Y)
			{		
				// 마지막 mls 가 마지막으로 사용한 디렉토리를 저장한다.	
				FILE *fp= fopen((conf.cfghome + "path").c_str(), "wb");
				
				if (fp)
				{
					fputs(panel->GetCurrentPath().c_str(), fp);
					fclose(fp);
				}
				else
				{
					LOG("'path' file write failure !!!");
				}
				g_MainFrame->Exit();
			}
		}
		
		void Menu() 	{	g_MainFrame->tMenu.Process(*panel);	}
		void Refresh()	{	panel->Refresh();		}
		
		void Mcd()
		{
			MLS::Mcd mcd;
			mcd.AddDirectory(panel->GetCurrentPath());
			mcd.setCur(panel->GetCurrentPath());
			if (mcd.proc()) 
			{
				panel->Read(mcd.getCur());
			}
			
			Refresh();
		}
			
		void Copy()
		{
			if (!panel->isZip()) panel->cmd_copy();
			else panel->cmd_zip_copy();
		}
		
		void Move()
		{
			if (!panel->isZip()) panel->cmd_move();
		}
		
		void Edit();   /// panel_cmd.cpp 에 구현
		void View();   /// panel_cmd.cpp 에 구현
			
		void Mkdir()
		{
			string p;
			if (InputBox("Make Directory", p)<0) return;
			
			struct stat statbuf;
			
			if (stat(p.c_str(), &statbuf)!=-1)
			{
				MsgBox("Error","Directory exists");
				return;				
			}
			
			if (mkdir(p.c_str(), 0755)==-1)
			{
				MsgBox("Error","Fail to make a directory");
				return;
			}
			Refresh();
			return;
		}
		
		void Rename()
		{
			struct stat statbuf;
			string ren = panel->GetCurrentFile().name;
				
			if (ren == "..") return;
				
			if (InputBox("rename", ren)<0) return;
			
			if (lstat(ren.c_str(), &statbuf) != -1)
			{
				MsgBox("Error", "fail : name exists");
				return;
			}
			
			if (rename(panel->GetCurrentFile().name.c_str(), ren.c_str()) == -1)
			{
				MsgBox("Error", "fail to rename");
				return;
			}
			Refresh();
			panel->Search(ren);
		}
		
		void ViewScreen();
		
		void Back()			{	panel->Read(panel->tHistory.GetPrev());	}
		void Forward()		{	panel->Read(panel->tHistory.GetPrev());	}
			
		void Select() {   panel->ToggleSelect(); }
		void ShowHidden()
		{
			conf.ShowHidden = !conf.ShowHidden;
			Refresh();
		}
		
		/// @brief	User로 부터 입력 받은 Shell 명령실행
		void Shell() 
		{ 
			string cmds;
			if (InputBox("Shell command", cmds))
			{
				ParseAndRun(cmds);
			}
			Refresh();
		}
		
}; // class CommandImp
	
}; // namespace 

#endif
