///	\file	conf.h	
///	\brief	config header
#ifndef __CONFIG_H__
#define __CONFIG_H__

#include <vector>
#include <list>
#include <string>
#include <map>

#include "drawutil.h"
#include "action.h"

namespace MLS {
	
/// \brief	config class
class Config
{
public:
	std::list<std::string> filter; ///< 필터
	std::string date_type,			///< date format 지정
			extern_viewer,			///< viewer 프로그램 지정 (vi)
			extern_editor;			///< editor 프로그램 지정(vi)
	std::string home,				///< home dir(~/)
			cfghome,				///< config home dir(~/.mls)
			mcdroot;				///< mcd root directory
	std::string colorset;			///< colorset(mls.col)
		
	// color
	color_entry MCD,				///< mcd clor_entry
				MCDstat,			///< MCDstat color entry?
				Dir,				///< Dir color entry
				Drive,				///< Drive color entry
				Line,				///< Line color entry?
				FuncN,				///< FuncN color entry?
				FuncA,				///< FuncA color entry?
				StatN,				///< StatN color entry?
				StatA,				///< StatA color entry?
				Else,				///< Else color entry?
				Help,				///< Help color entry?
				Quick,				///< Quick color entry?
				QuickMCD,			///< QuickMCD color entry?
				Dialog;				///< Dialog color entry?
	
	std::map<std::string, color_entry> name_col;	///< name_col
	std::map<std::string, color_entry> ext_col;		///< ext_col
	std::map<int, color_entry> mask_col;			///< mask_col
	
	std::map<std::string, Action> ext_prog;			///< ext prog

	Action Func[12];  ///< F1~F12
	Action McdFunc[12];
	Action aFunc[12]; ///< F1~F12
	Action cFunc[12]; ///< F1~F12
	
	// toggle
	bool ShowHidden;	///< .으로 시작하는 파일 보이기..
	bool AutoDrawmode;  ///< 그리기 방식을 자동으로 선택.
	bool FixedRow;      ///< 고정된 세로줄
	bool DirSort;		///< DirSort
	bool McdSort;		///< McdSort

	int sort;           ///< 정렬방식
	int column;      ///< 그리기 방식(컬럼수)

	Config();
	~Config();

	void set_default();
	int  load(const char *);
	int  load_colorset(const char *);
};

extern Config conf;			///< Config extern 변수

}; // namespace

#endif
