///  @file		main.cpp
///  @brief 	main source file
///  @author	mls programer
///  @date		2004-

#include <stdlib.h>
#include <ncurses.h>

#include <signal.h>
#include <sys/types.h>
#include <sys/wait.h>

#include "define.h"
#include "mainframe.h"
#include "mls.h"

#include "panel.h"
#include "mcd.h"

#include "conf.h"
#include "mlslocale.h"

using namespace MLS;

char *logfile ="/tmp/log.txt";	///< log file

/// @brief 	시그널 처리를 하는 함수
/// @param sig	시그널 Number
void signal_action(int sig)
{
	refresh();
	LOG("Signal :: %d", sig);
}

/// @brief 시그널블럭 처리를 한다.
/// @return SUCCESS(0)
int signal_blocking()
{
	// Ctrl+C 를 막아주는 코드
	struct sigaction act;
	
	// 다음은 시그널이 들어왔을 때 실행 함수
	//act.sa_handler = signal_action;
	
	// 시그널 무시
	act.sa_handler = SIG_IGN;
	sigemptyset(&act.sa_mask);
	act.sa_flags = 0;
	
	sigaction(SIGINT, &act, 0);
	sigaction(SIGQUIT, &act, 0);
	return SUCCESS;
}


/// @brief	mls 시작시 초기화 함수
///
/// 주로 nCurses 초기화를 한다.
///
/// @return	성공여부 0일때 성공
int init()
{	
	// nCurses 설정
	initscr();
	// nCurses : Color 시작
	start_color();

	// . 엔터 처리?
	nonl();

	// . nCurses : 즉시 입력이 가능하도록 설정
	cbreak();

	// . nCurses : echo모드 해제
	noecho();	

	// . keypad 활성화
	keypad(stdscr, TRUE);

	// . 색깔 쌍을 초기화 한다. 0 ~ 49개 까지
	for (int t=0; t<8; t++)
		for (int t2=0; t2<8; t2++)
		{
			if (t==0 && t2==0) continue;
			init_pair(t*8+t2, t, t2);
		}
	return 0;
}

/// @brief	mls가 종료될때 호출되는 함수
void destroy()
{
	// . keypad 비활성화
	keypad(stdscr, FALSE);
	// . 라인단위로 입력이 이루어지도록 설정
	nocbreak();
	
	noecho();
	// . nCurses 해제
	endwin();
}

// 설정화일

/// @brief	진입부
/// @param	argc	인자 개수
/// @param	argv	인자
/// @return	성공여부
int main(int argc, char *argv[])
{
	// Ctrl+C 를 막는다.
	signal_blocking();
	
	// . 시스템 초기화	
	if (init()) return 1;
	
	setlocale(LC_ALL, getenv("LANG"));
	
	// . Locale  설정
	set_locale(US);
	
	g_MainFrame = new MainFrame;
	g_MainFrame->Do();
	
	destroy();
	
	return SUCCESS;
}
