/**
 * @file	mainframe.cpp
 * @brief	mainframe source file 
 * @author	la9527
 * @date	2004-08-22
 */
#include "define.h"
#include "mainframe.h"
#include "mlslocale.h"
#include "dialog.h"

namespace MLS {

MainFrame *g_MainFrame;

MainFrame::MainFrame()
{
	bSplit  = false;
	nActive = 0;
}

MainFrame::~MainFrame()
{	
}

void MainFrame::SetSplitWindow( )
{
	if (bSplit)
	{
		tPanel[0].SetWindow(win1, (COLS-2)/2, LINES-6);
		tPanel[1].SetWindow(win2, (COLS-1)/2, LINES-6);		
	}
	else
	{
		tPanel[nActive].SetWindow(win0, COLS, LINES-6);	
	}	
}


void MainFrame::Draw()
{
	int i;
	// . Function Panel Draw
	setcol(conf.FuncN);
	move(0,0);
	hline(' ', COLS);
	for (i=0; i<8; i++)
		mvprintw(0, 2+i*(COLS/8),"%s", conf.Func[i+1].getName());
	unsetcol();
	
	setcol(conf.FuncA);
	mvprintw (0, 0, "F");
	for (i=0; i<8; i++)
		mvprintw(0, 1+i*(COLS/8), "%d", i+2);
	unsetcol();	
	
	if (conf.FuncN.back >=8 ) attron(A_BLINK);
	attron(COLOR(COLOR_BLACK, conf.FuncN.back%8));
	for (i=1; i<8;i++)
		mvaddch(0, i*(COLS/8), VLINE);
	unsetcol();

	// . PATH Draw
	setcol(conf.StatN);
	move(1, 0);
	hline(' ', COLS);
	printw("%s", tPanel[nActive].GetCurrentPath().c_str());	
	unsetcol();	
	
	// 위 아래 선 및 경계선 긋기..
	setcol(conf.Line);	
	move(2,0);	hline(HLINE, COLS);	
	move(LINES-3,0);	hline(HLINE, COLS);
	
	// 2단일 경우..	
	if (bSplit)
	{
		for (int t=0; t<LINES-6; t++)
		{
			mvaddch(3+t, (COLS-2)/2, VLINE);	
			addch(VLINE);
		}
		mvaddch(2, (COLS-2)/2, TTEE);
		addch(TTEE);
		mvaddch(LINES-3, (COLS-2)/2, BTEE);
		addch(BTEE);
	}
	
	//
	unsetcol();
	
	if (bSplit)
	{
		tPanel[0].draw();
		tPanel[1].draw();		
	}
	else
	{
		tPanel[nActive].draw();
	}
	
	tPanel[nActive].DrawStatus();
	
	refresh();
}

///	@brief 메인프레임 수행
void MainFrame::Do()
{	
	win0 = newwin(LINES-6, COLS, 3, 0);
	win1 = newwin(LINES-6, (COLS-2)/2, 3, 0);
	win2 = newwin(LINES-6, (COLS-1)/2, 3, (COLS+2)/2);
	
	bExit = false;
	
	// 메뉴
	tMenu.Create();
	
	tKeybind.Load("/etc/mls/mls.key");
	tKeybind.Load(conf.cfghome + "mls.key");
	
	
	tCommand.Init();
		
	tPanel[0].Read(".");
	tPanel[1].Read(".");	
	tCommand.SetPanel(&tPanel[0]);
	
	SetSplitWindow();
	
	refresh();
	
	// ESC 딜레이 줄이기..	
	ESCDELAY = 10;
	
	while(!bExit)
	{
		// 패널 화면을 그린다.
		Draw();
	
		// 키를 입력 받는다.
		int nKey = getch();
		
		if (nKey == KEY_RESIZE)
		{
			wresize(win0, LINES-6, COLS);
			wresize(win1, LINES-6, (COLS-2)/2);
			wresize(win2, LINES-6, (COLS-1)/2);
			
			mvwin(win1, 3, 0);
			mvwin(win2, 3, (COLS+2)/2);
			
			SetSplitWindow();
			
			continue;			
		}
		
		if (nKey == '1')
		{
			bSplit = !bSplit;
			SetSplitWindow();
			continue;
		}
		
		if (bSplit && nKey == '\t')
		{
			nActive = (nActive + 1) %2;
			tCommand.SetPanel(&tPanel[nActive]);
			continue;
		}
				
		// Searching..
		if (tPanel[nActive].SearchProcess(nKey)) continue;
		
		// Function key
		if (KEY_F(1) <= nKey && nKey <=KEY_F(12))
		{
			if (conf.Func[nKey-KEY_F(1)].size()>=1)
				tCommand.Execute((conf.Func[nKey-KEY_F(1)])[0]);
			continue;
		}
		
		// Execute		
		if (nKey != 27)
			tCommand.Execute(tKeybind.GetCommand(nKey));
		else
		{
			// Alt + 문자는 이렇게 받는다. 
			nodelay(stdscr, TRUE);
			nKey = getch();
			nodelay(stdscr, FALSE);
			
			if (nKey != ERR)
			{
				tCommand.Execute(tKeybind.GetMetaCommand(nKey));
			}
			else
			{
				tCommand.Execute("<viewscreen>");
			}
		}
	}
}


}; // namespace 
