///	\file	mcd.h
///	\brief	mcd.h
#ifndef MLSmcd_H__
#define MLSmcd_H__

#include <string>
#include <vector>
#include <unistd.h>
#include <ncurses.h>

#include "mls.h"

namespace MLS {

///	\brief	mcd class
class Mcd
{
	
protected:

	///	\brief	directory 정보 
	struct dir
	{
		std::string name;		///< directory name
		int  		depth,		///< directory 깊이
					row,		///< directory ?
					index;		///< index?
		struct dir  *parent;	///< parent directory
		int 		nlink;		///< link?
		bool        linked;		///< linked ?

		std::vector<dir*> node;	///< node
		
		///	\brief	생성자
		///	\param	p		directory name
		///	\param	pa		parent directory
		///	\param	n		nlink
		///	\param	l		linked
		dir(std::string p, dir *pa, int n, bool l = false) 
			: name(p), parent(pa), nlink(n), linked(l) {depth = pa ? pa->depth+1 : 0;}
		
		std::string &path();
		
	};
	
	///	\brief	mcdsort
	class mcdsort
	{
	public:
		///	\brief	operator() 함수
		///	\param	a	directory 정보
		///	\param	b	directory 정보
		///	\return	true :		a directory 명이 b directory 명보다 우선 정렬
		///			false:		b directory 명이 a directory 명보다 우선 정렬
		bool operator()(dir *a, dir *b)	{return a->name < b->name;}
	};
	
	WINDOW 			*win;
	
	typedef std::vector<dir *>	         pDirContainer;	///< dir * 벡터 재정의
	typedef std::vector<dir *>::iterator pDirIterator;///< dir * iterator 재정의
	
	pDirContainer order;  		///< 순회 테이블
	pDirIterator  cur;			///< 현 위치 index
	
	dir           *root;        ///< 루트 트리..  마지막 '/' 을 뺀 이름만을 기록
	int           sRow,			///< 스크롤 한 Row	
				  sCol;			///< 스크롤 한 Col	
	bool          bExit;		///< exit 여부
	std::string   strSearch;    ///< 검색단어
	std::string   infofile;		///< treeinfo 파일 위치
	char          *code;        ///< treeinfo 헤더
	const char          *title;	///< title string
	
	// 순회테이블 재정리, cur 무효화
	void setOrder();
	
	// 커서이동	
	void left();
	void right();
	void up();
	void down();
	
	// scan fuction	
	int  load();
	int  save();
	int  rescan();	
	int  scan(dir *, bool = true);
	void destroy(dir *, bool = true);	
	
	// draw function
	void draw();

public:	
	Mcd(const char * = "MCD - Mls Change Directory");
	~Mcd();
	
	void setCur(const std::string &);

	///	\brief	현재 위치한 정보를 얻어낸다.
	///	\return	현재 위치한 정보
	std::string &getCur() {return (*cur)->path();}
	void AddDirectory(const std::string &);
	
	int Search(const std::string &str);
	bool proc();
};

}; // namespace

#endif //MLSmcd_H__
