///	\file	mcd_draw.cpp
///	\brief	mcd_draw.cpp
#include <curses.h>
#include <vector>

#include "mls.h"
#include "mcd.h"
#include "conf.h"
#include "mlslocale.h"
#include "drawutil.h"

using namespace std;
using namespace MLS;

int col_location[6]={0, 4, 18, 32, 46, 60};	///< 화면에 그릴 tree 깊이(x위치)

///	\brief	mcd draw 함수
void Mcd::draw()
{
	vector<chtype> line;
	dir *node;
	dir *pcur = *cur;
	int row, col, odep=0;	
	int i;
	
	if (pcur->row - sRow > LINES - 5) sRow = pcur->row - LINES +5;
	if (pcur->depth - sCol > 5) sCol = pcur->depth - 5;
	if (pcur->row - sRow < 0) sRow = pcur->row;
	if (pcur->depth - sCol < 1)
	{
		 sCol = pcur->depth-1;	
		 if (sCol == -1) sCol=0;		 
	}
	
	// 화면 지우기	
	setcol(conf.MCD, win);
	for (int j=0;j<LINES; j++)
	{
		wmove(win, j, 0);
		whline(win, ' ', COLS);
	}
	unsetcol(win);
	
	setcol(conf.MCDstat, win);
	wmove(win, 0, 0);
	whline(win, ' ', COLS);
	mvwprintw(win, 0, 0, "%s", title);
	unsetcol(win);
	
	setcol(conf.FuncN, win);
	wmove(win, LINES-1,0);
	whline(win, ' ', COLS);
	for (i=0; i<8; i++)
		mvwprintw(win, LINES-1, 2+i*(COLS/8),"%s", conf.McdFunc[i+1].getName());
	unsetcol(win);
	
	setcol(conf.FuncA, win);
	mvwprintw (win, LINES-1, 0, "F");
	for (i=0; i<8; i++)
		mvwprintw(win, LINES-1, 1+i*(COLS/8), "%d", i+2);
	unsetcol(win);	
	
	if (conf.FuncN.back >=8 ) attron(A_BLINK);
	attron(COLOR(COLOR_BLACK, conf.FuncN.back%8));
	for (i=1; i<8;i++)
		mvwaddch(win, LINES-1, i*(COLS/8), VLINE);
	unsetcol(win);
		
	// 타이틀
	/*
	setcol(conf.MCDstat);
		mvprintw(0, 0, "%-80.80s", title);
		mvprintw(LINES-1, 0, "F1-Help|F2-Rescan|F3-Drive|F4-Move|F5-Copy|F6-Ren|F7-Make|F8-Del|F9-Size|F11-QCD");
	unsetcol();		
	*/
	
	if (!strSearch.empty())
	{
		setcol(conf.MCD, win);
		mvwprintw(win, 1, 0, "Search : %s", strSearch.c_str());
		unsetcol(win);
	}
	
	for (pDirIterator i=order.begin(); i!=order.end(); i++)
	{
		node = *i;
				
		if (odep < node->depth)
			line.push_back(VLINE);
		else
		while(node->depth < odep)
		{
			odep--;
			line.pop_back();
		}
		
		odep = node->depth;
		col  = node->depth;
		row  = node->row;
				
		// 출력 다했으면 종료		
		if (node->depth != 0 && node->parent->node.back()==node)
			line[col-1]=' ';
			
		if (row - sRow > LINES - 5) break;
		if (row - sRow < 0) continue;
		
		setcol(conf.MCD, win);
		if (node != root && node->parent->node.front()!=node) 
			for (int t = sCol; t<col && t<sCol+5; t++)
				mvwaddch(win, row-sRow+2, col_location[t-sCol+1], line[t]);			
		unsetcol(win);
		
		if (col - sCol > 5 ) continue;
		if (sCol != 0 && col - sCol < 1 ) continue;
						
		if (node->depth==0)
		{// 루트라면
			if (node == pcur) setrcol(conf.MCD,win);
			else setcol(conf.MCD,win);
			
			wmove(win, row-sRow+2, 2);
			waddch(win, '/');	
			unsetcol(win);
		}		
		else		
		{// 아니라면
			setcol(conf.MCD);
			
			if (node->parent->node.size() > 1)
			{// 디렉토리가 하나가 아닐때 
				if (node->parent->node.front()==node)
				{// 첫번째 노드 
					wmove(win, row-sRow+2, col_location[col-sCol]);					
					waddch(win, TTEE);
				}
				else
				{// 그 이외의 디렉토리 
					wmove(win, row-sRow+2, col_location[col-sCol]);
										
					if (node->parent->node.back()==node) waddch(win, LLCORNER);
					else waddch(win, LTEE);
				}	
			}
			else 
			{
				wmove(win, row-sRow+2, col_location[col-sCol]);
				waddch(win, HLINE);
			}
			waddch(win, HLINE);
			unsetcol(win);
						
			// 선을 하나 더 긋고 이름 출력 
			if (node == pcur) 
			{
				setrcol(conf.MCD, win);
				if (node->name.size() <= 12)
					wprintw(win, "%-12s", node->name.c_str());
				else 
					wprintw(win, "%-11.11s~", node->name.c_str());
			}				
			else
			{
				if (node->linked) setcol(11, conf.MCD.back, win);
				else setcol(conf.MCD, win);
				if (node->name.size() <= 12)
				{
					wprintw(win, "%s", node->name.c_str());
					if (!node->node.empty())
						whline(win, HLINE, 12-node->name.size());
				}
				else wprintw(win, "%-11.11s~", node->name.c_str());
			}			
			unsetcol(win);
		}
	}
	
	setcol(conf.MCD, win);
	mvwprintw(win, LINES-2, 0, " Path %s", (*cur)->path().c_str());
	unsetcol(win);
	
	wrefresh(win);
	refresh();
}
