///	@file	panel.cpp
///	@brief	panel.cpp
#include <ncurses.h>
#include <map>
#include <sys/stat.h>

#include "mls.h"
#include "panel.h"
#include "mcd.h"
#include "conf.h"
#include "mlslocale.h"
#include "strutil.h"
#include "keydef.h"
#include "mainframe.h"
#include "dialog.h"

using namespace strutil;
using namespace MLS;

/// @brief	생성자
Panel::Panel()
{
	cur=0;
	page=0;
	
	sel_size=0;
	sel_num=0;
	
	// Zip 정의
	m_nZip = 0;
	m_sZipFile = "";
	
	bExit=false;	
	
	lines = LINES;
	cols = COLS;
}

/// @brief	소멸자
Panel::~Panel()
{
	delwin(win);
}

/// @brief	비슷한 이름을 가진 파일을 찾는다.
///	@param	str		찾을 파일 앞자리나 전체 파일명
///	@return	찾은 파일 index
int 
Panel::Search(const string &str)
{
	int l = file_list.size();
	
	string p=tolower(str);
	
	for (int t=0; t < l ; t++)
	{
		if ( file_list[t]->name.size() < str.size() ) continue;
		if ( tolower(file_list[t]->name.substr(0, str.size())) == p)
		{
			cur = t;
			return t;
	 	}
	}	
	// 없으면
	return -1;	
}


/// @brief 엔터키
void
Panel::Key_Enter()
{
	int key, bkey;
	bool bSearch = false;
	
	string tTmp;
	string::size_type	tSizeType;
	
	if (m_nZip == 0)
	{
		LOG("Read :: %s", file_list[cur]->name.c_str());
		if (Read(file_list[cur]->name))
		{
			LOG("Read Success");
			return;
		}
	}
	else
	{
		if (file_list[cur]->FullName == "/")
		{
			// new된 Zip 을 삭제
			m_tReader->Destroy();
			m_nZip = 0;
			
			Read(GetCurrentPath());
			return;
		}
		
		if (file_list[cur]->bDir)
		{
			// 압축 파일은 FullName 으로 한다.
			LOG("Read Zip :: %s", file_list[cur]->FullName.c_str());
			if (file_list[cur]->name == "..") // .. 구현
			{
				tTmp = file_list[cur]->FullName;
				tTmp = tTmp.substr(0, tTmp.size() - 1);
				
				tSizeType = tTmp.rfind("/");
				tTmp = tTmp.substr(0, tSizeType+1);
				
				m_zipPath = m_sZipFile + " :: /" + tTmp;
				if (Read(tTmp)) return;
			}
			else
			{
				m_zipPath = m_sZipFile + " :: /" + file_list[cur]->FullName;
				if (Read(file_list[cur]->FullName)) return;
			}
			return;
		}
	}
		
	if (!file_list[cur]->bDir)
	{
		WINDOW*	pWin = NULL;
		
		if (m_nZip == 0)
		{
			m_nZip = 1;
			pWin = MsgWaitBox("Wait", "Please wait !!!");
			
			m_sZipFile = file_list[cur]->name;
			
			m_zipPath = m_sZipFile + " :: /";
			if (Read("") == true)
			{
				if (pWin) MsgWaitEnd(pWin);
				return;
			}
			else
			{
				m_nZip = 0;
				if (pWin) MsgWaitEnd(pWin);
				
				// 파일 실행	
				std::map<std::string, Action>::iterator i	= conf.ext_prog.find( tolower( file_list[cur]->ext()) );
				
				if (i != conf.ext_prog.end())
				{
					if ((*i).second.size()==1)	g_MainFrame->tCommand.Execute(((*i).second)[0]);
				}
				else if (file_list[cur]->attr[3] == 'x')
				{
					std::string cmd;
					cmd = "./" + file_list[cur]->name;
					
					destroy();
					system(cmd.c_str());
					init();
				}
			}
		}
		else
		{
			string	sTmp;
			File*	pFile	= file_list[cur];
			
			sTmp = pFile->name;
			
			pWin = MsgWaitBox("Wait", "Please wait !!!");
			
			// 압축 파일 /tmp에 풀고 filename 리턴
			LOG("DataName :: [%s]", sTmp.c_str());
			if (m_tReader->View(*pFile, "/tmp") == SUCCESS)
			{
				if (pWin) MsgWaitEnd(pWin);
				pFile->name = "/tmp/" + pFile->FullName;
			
				// 파일 실행
				std::map<std::string, Action>::iterator i = conf.ext_prog.find( tolower( pFile->ext()) );
				
				if (i != conf.ext_prog.end())
				{
					if ((*i).second.size()==1)
						g_MainFrame->tCommand.Execute(((*i).second)[0]);						
				}
				else if (pFile->attr[3] == 'x')
				{
					destroy();
					system(pFile->name.c_str());
					init();
				}
				remove(pFile->name.c_str());
				pFile->name = sTmp;
			}
			else
			{
				pFile->name = sTmp;
				if (pWin) MsgWaitEnd(pWin);
				MsgBox("ERROR", "file reading failure !!!!");
			}
			Read("");
			return;
		}
	}
}

/// @brief	왼쪽 방향키
void
Panel::Key_Left()
{
	if (cur < (unsigned)nrow) 
		cur=0;
	else 
		cur-=nrow;
}

/// @brief	오른쪽 방향 키
void
Panel::Key_Right()
{
	cur+=nrow;
	if (cur > file_list.size()-1) cur=file_list.size()-1;
}

/// @brief	위쪽 방향키
void
Panel::Key_Up()
{
	if (cur > 0) cur--;
}

/// @brief	아래쪽 방향키
void
Panel::Key_Down()
{
	if (cur < file_list.size()-1) cur++;
}

/// @brief	page up key
void
Panel::Key_PageUp()
{
	if (cur < (unsigned)nrow*ncol) cur=0;
			else cur-=(nrow*ncol);
}

/// @brief	page down key
void
Panel::Key_PageDown()
{
	cur+=(nrow*ncol);
	if (cur > file_list.size()-1) cur=file_list.size()-1;
}

/// @brief	home key
void
Panel::Key_Home()
{
	cur = 0;
}

/// @brief	Key End
void
Panel::Key_End()
{
	cur = file_list.size()-1;
}

/// @brief  키를 입력받아 파일 찾기
/// @param 
/// @return 찾았는가 못찾았는가
bool
Panel::SearchProcess(int key)
{
	static bool bSearch = false;
	
	if (!bSearch && ( isalnum(key) || key == '.' ) ) bSearch = true;
		
		// .. 파일을 찾아 커서를 파일위에 둔다
	if (bSearch &&  ( (32<= key && key <=126) || key == KEY_BS ) )
	{
		int tmp;
		// search mode;
		// 뒷부분에 key를 넣고
		
		if (key == KEY_BS) 
		{
			strSearch.erase(strSearch.size()-1, 1);
			if (strSearch.empty()) {bSearch = false; return false;}
		}
		else strSearch += (char) key;
		
		// strSearch로 시작하는 문자를 찾는다..						
		if ((tmp = Search(strSearch))== -1)		
		{			
			// 없으면 처음에 넣고 다시 찾는다.				
			if ((tmp = Search(string(1, key)))!= -1) strSearch = (char)key;
			else strSearch.erase(strSearch.size()-1, 1);
		}		
		return true;
	}
	
	bSearch = false;
	strSearch.erase();
	
	return false;
}

/// @brief	파일선택
void
Panel::ToggleSelect()
{
	if (file_list[cur]->selected) deselect(*file_list[cur]);
	else select(*file_list[cur]);
			
	if (cur < file_list.size()-1) cur++;			
}

///	@brief	panel 처리

/*
void 
Panel::proc()
{
	int key, bkey;
	bool bSearch = false;
	
	string tTmp;
	string::size_type	tSizeType;		
	
	// . 끝내기 전까지 무한 반복한다.
	while(!bExit)
	{			
			
		case '*':	// 모두선택이나 모두 선택반전
			for (pFileIterator i=file_list.begin();i!=file_list.end(); i++)
			{
				if ((*i)->selected) deselect(*(*i));
				else select(*(*i));
			}
			break;

	
		case KEY_CTRL_P:	// Ctrl + p Key
		{
			int size;
			vector<string> &p = GetSelection(size, true);
			
			for (vector<string>::iterator i=p.begin(); i!=p.end(); ++i)
				LOG("%s", (*i).c_str());
			break;
		}		
			
					
		case KEY_BS:
			Read("..");
			break;		
			
		case '/':
		{
			string cmds;
			if (InputBox("Shell command", cmds)<0) break;
			Command(cmds);
			break;
		}
			
		case '\\':
			Read("/");
			break;
			
		case 27: // ESC-meta, alt
			bkey = getch();
			
			switch(bkey)
			{
			case 27: // esc 두번 누르면 Zip은 종료 된다.
				if (m_nZip == 1)
				{	
					// new된 Zip 을 삭제
					m_tReader->Destroy();
					m_nZip = 0;
					if (Read(GetCurrentPath()))
					{
						break;
					}
				}
				break;
				
			case '0':	// 자동 칼럼 모드
				conf.AutoDrawmode = !conf.AutoDrawmode;
				break;
			case '1':	// 1 칼럼 모드
				conf.column = 1;
				conf.AutoDrawmode=false;
				break; 
			case '2':	// 2 칼럼 모드
				conf.column = 2;
				conf.AutoDrawmode=false;
				break;
			case '3':	// 3 칼럼 모드
				conf.column = 3;
				conf.AutoDrawmode=false;
				break;
			case '4':	// 4 칼럼 모드
				conf.column = 4;
				conf.AutoDrawmode=false;
				break;
			case '5':	// 5 칼럼 모드
				conf.column = 5;
				conf.AutoDrawmode=false;
				break;
			case '6':	// 6 칼럼 모드
				conf.column = 6;
				conf.AutoDrawmode=false;
				break;
				
			case 'N':	// 이름별로 파일정렬
			case 'n':
				conf.sort = 1;
				Sort();
				break;
				
			case 'E':	// 확장자 별로 파일정렬
			case 'e':
				conf.sort = 2;
				Sort();
				break;
				
			case 'S':	// 파일 사이즈별로 파일정렬
			case 's':
				conf.sort = 3;
				Sort();
				break;
				
			case 'T':	// 생성 날짜별로 파일정렬
			case 't':
				conf.sort = 4;
				Sort();
				break;
			case 'B':	// ?정렬
			case 'b':
				conf.sort = 5;
				Sort();
				break;
				
//----------------------------
				
			case 'D':
			case 'd':
				// 압축 파일이 지워지면 다시 읽어들어야 되기 때문.
				if (m_nZip == 1) 
				{
					cmd_zip_remove();
					m_tReader->Destroy();
					
					m_nZip = 0;
					if (Read(GetCurrentPath()))
					{
						break;
					}
				}
				else
				{
					Command("<del>");
				}
				break;				
			
				
				
			default:			
				break;
			}
		}
	}
	
	// 마지막 mls 가 마지막으로 사용한 디렉토리를 저장한다.	
	FILE *fp= fopen((conf.cfghome + "path").c_str(), "wb");
	
	if (fp)
	{
		//fputs(path.c_str(), fp);
		fputs(GetCurrentPath().c_str(), fp);
		fclose(fp);
	}
	else
	{
		LOG("'path' file write failure !!!");
	}
}
*/
void Panel::SetWindow( WINDOW *_win, int w, int h)
{
	win = _win;
			
	lines=h;
	cols=w;
}
